@@ -37,7 +37,7 @@ scalaConfig <- function(verbose=TRUE, reconfig=FALSE, download=character(0), req
   download.sbt <- "sbt" %in% download
   if ( Sys.getenv("RSCALA_RECONFIG") != "" ) reconfig <- Sys.getenv("RSCALA_RECONFIG")
   consent <- identical(reconfig,TRUE) || download.java || download.scala || download.sbt
-  installPath <- file.path("~",".rscala")
+  installPath <- path.expand(file.path("~",".rscala"))
   dependsPath <- if ( Sys.getenv("RSCALA_BUILDING") != "" ) file.path(getwd(),"inst","dependencies") else ""
   offerInstall <- function(msg) {
     if ( !identical(reconfig,"live") && interactive() ) {
@@ -53,7 +53,7 @@ scalaConfig <- function(verbose=TRUE, reconfig=FALSE, download=character(0), req
   if ( identical(reconfig,FALSE) && file.exists(configPath) && !download.java && !download.scala && !download.sbt ) {
     if ( verbose ) cat(paste0("\nRead existing configuration file: ",configPath,"\n\n"))
     source(configPath,chdir=TRUE,local=TRUE)
-    if ( is.null(config$format) || ( config$format < 3L ) || ( ! all(file.exists(c(config$javaHome,config$scalaHome,config$javaCmd,config$scalaCmd))) ) || ( is.null(config$sbtCmd) && require.sbt ) || ( ! is.null(config$sbtCmd) && ! file.exists(config$sbtCmd) ) ) {
+    if ( is.null(config$format) || ( config$format < 4L ) || ( ! all(file.exists(c(config$javaHome,config$scalaHome,config$javaCmd,config$scalaCmd))) ) || ( is.null(config$sbtCmd) && require.sbt ) || ( ! is.null(config$sbtCmd) && ! file.exists(config$sbtCmd) ) ) {
       if ( verbose ) cat("The 'config.R' is out-of-date.  Reconfiguring...\n")
       unlink(configPath)
       scalaConfig(verbose, reconfig, download, require.sbt)
@@ -101,17 +101,13 @@ scalaConfig <- function(verbose=TRUE, reconfig=FALSE, download=character(0), req
       consent <- consent || consent2
     }
     osArchitecture <- if ( javaConf$javaArchitecture == 32 ) {
-      isOS64bit <- if ( identical(.Platform$OS.type,"windows") ) {
-        out <- system2("wmic",c("/locale:ms_409","OS","get","osarchitecture","/VALUE"),stdout=TRUE)
-        any("OSArchitecture=64-bit"==trimws(out))
-      } else identical(system2("uname","-m",stdout=TRUE),"x86_64")
-      osArchitecture <- if ( isOS64bit ) 64 else 32
+      osArchitecture <- if ( isOS64bit() ) 64 else 32
       if ( osArchitecture == 64 ) {
         warning("32-bit Java is paired with a 64-bit operating system.  To access more memory, please run 'scalaConfig(download=\"java\")'.")
       }
       osArchitecture
     } else javaConf$javaArchitecture
-    config <- c(format=3L,osArchitecture=osArchitecture,scalaConf,javaConf)
+    config <- c(format=4L,osArchitecture=osArchitecture,scalaConf,javaConf)
     if ( download.sbt ) installSBT(installPath,config,verbose)
     sbtSpecifics <- function(x,y) list(sbtCmd=x)
     sbtConf <- findExecutable("sbt","SBT",installPath,sbtSpecifics,verbose)
@@ -165,6 +161,13 @@ findExecutable <- function(mode,prettyMode,installPath,mapper,verbose=TRUE) {  #
   conf <- tryCandidate(file.path(installPath,candidates))
   if ( ! is.null(conf) ) return(conf)
   ###
+  if ( mode == "java" ) {
+    label <- paste0("R CMD config JAVA")
+    path <- tryCatch(system2(file.path(R.home("bin"),"R"),c("CMD","config","JAVA"),stdout=TRUE,stderr=FALSE), warning=function(e) NULL, error=function(e) NULL)
+    conf <- tryCandidate(path)
+    if ( ! is.null(conf) ) return(conf)
+  }
+  ###
   label <- paste0(allCaps,"CMD environment variable")
   conf <- tryCandidate(Sys.getenv(paste0(allCaps,"CMD")))
   if ( ! is.null(conf) ) return(conf)
@@ -191,12 +194,21 @@ findExecutable <- function(mode,prettyMode,installPath,mapper,verbose=TRUE) {  #
   NULL
 }
 
+isOS64bit <- function() {
+  if ( identical(.Platform$OS.type,"windows") ) {
+    out <- system2("wmic",c("/locale:ms_409","OS","get","osarchitecture","/VALUE"),stdout=TRUE)
+    any("OSArchitecture=64-bit"==trimws(out))
+  } else identical(system2("uname","-m",stdout=TRUE),"x86_64")
+}
+      
 getURL <- function(software, version, attempt=1, counter=1) {
   stub <- if ( counter == 1 )       sprintf("https://raw.githubusercontent.com/dbdahl/rscala")
           else if ( counter == 2 )  sprintf("https://dahl-git.byu.edu/dahl/rscala/raw")
           else return(NULL)
   url <- if ( software == "java" ) {
     os <- osType()
+    if ( ! ( os %in% c("windows","mac","linux") ) ) stop(sprintf("Cannot automatically download Java for %s.",osType))
+    if ( ! isOS64bit() ) stop(sprintf("Cannot automatically download Java for a 32-bit operating system.",osType))
     sprintf("%s/master/url/java/%s/%s/%s",stub,os,version,attempt)
   } else if ( software == "scala" ) {
     sprintf("%s/master/url/scala/%s/%s",stub,version,attempt)   
@@ -211,6 +223,7 @@ installJava <- function(installPath, verbose, attempt=1) {
   if ( verbose ) cat("\nDownloading Java.\n")
   dir.create(installPath,showWarnings=FALSE,recursive=TRUE)
   version <- Sys.getenv("RSCALA_JAVA_MAJORVERSION","8")
+  attempt <- as.integer(Sys.getenv("RSCALA_JAVA_ATTEMPT",attempt))
   url <- getURL("java",version,attempt)
   installPathTemp <- file.path(installPath,"tmp")
   unlink(installPathTemp,recursive=TRUE,force=TRUE)
@@ -236,7 +249,7 @@ installJava <- function(installPath, verbose, attempt=1) {
   unlink(destdir,recursive=TRUE,force=TRUE)  # Delete older version
   javaHome <- list.files(installPathTemp,full.names=TRUE,recursive=FALSE)
   javaHome <- javaHome[dir.exists(javaHome)]
-  if ( length(javaHome) != 1 ) stop(paste0("Problem extracting Java.  Delete the directory '",path.expand(installPath),"' and try again."))
+  if ( length(javaHome) != 1 ) stop(paste0("Problem extracting Java.  Delete the directory '",installPath,"' and try again."))
   file.rename(javaHome,destdir)
   unlink(installPathTemp,recursive=TRUE,force=TRUE)
   if ( verbose ) cat("Successfully installed Java at ",destdir,"\n",sep="")
@@ -249,10 +262,11 @@ installScala <- function(installPath, javaConf, verbose, attempt=1) {
   unlink(file.path(installPath,"scala"),recursive=TRUE)  # Delete older version
   majorVersion <- Sys.getenv("RSCALA_SCALA_MAJORVERSION","2.12")
   if ( javaConf$javaMajorVersion <= 7 ) majorVersion <- "2.11"
+  attempt <- as.integer(Sys.getenv("RSCALA_SCALA_ATTEMPT",attempt))
   url <- getURL("scala",majorVersion,attempt)
   dir.create(installPath,showWarnings=FALSE,recursive=TRUE)
   destfile <- file.path(installPath,basename(url))
-  result <- tryCatch( utils::download.file(url, destfile), error=function(e) 1, warning=function(e) 1)
+  result <- tryCatch(utils::download.file(url, destfile), error=function(e) 1, warning=function(e) 1)
   if ( result != 0 ) {
     unlink(destfile)
     msg <- "Failed to download installation."
@@ -268,7 +282,7 @@ installScala <- function(installPath, javaConf, verbose, attempt=1) {
   if ( result == 0 ) {
     destdir <- file.path(installPath,"scala")
     scalaHome <- list.files(installPath,sprintf("^scala-%s",majorVersion),full.names=TRUE)
-    if ( length(scalaHome) != 1 ) stop(paste0("Problem extracting Scala.  Delete the directory '",path.expand(installPath),"' and try again."))
+    if ( length(scalaHome) != 1 ) stop(paste0("Problem extracting Scala.  Delete the directory '",installPath,"' and try again."))
     file.rename(scalaHome,destdir)
     if ( verbose ) cat("Successfully installed Scala at ",destdir,"\n",sep="")   
   } else {
@@ -283,10 +297,11 @@ installSBT <- function(installPath, javaConf, verbose, attempt=1) {
   dir.create(installPath,showWarnings=FALSE,recursive=TRUE)
   unlink(file.path(installPath,"sbt"),recursive=TRUE)  # Delete older version
   # if ( javaConf$javaMajorVersion != 8 ) stop("Java 8 is recommended for SBT.")
+  attempt <- as.integer(Sys.getenv("RSCALA_SBT_ATTEMPT",attempt))
   url <- getURL("sbt",version,attempt)
   dir.create(installPath,showWarnings=FALSE,recursive=TRUE)
   destfile <- file.path(installPath,basename(url))
-  result <- tryCatch( utils::download.file(url, destfile), error=function(e) 1, warning=function(e) 1)
+  result <- tryCatch(utils::download.file(url, destfile), error=function(e) 1, warning=function(e) 1)
   if ( result != 0 ) {
     unlink(destfile)
     msg <- "Failed to download installation."
@@ -311,7 +326,7 @@ installSBT <- function(installPath, javaConf, verbose, attempt=1) {
 
 javaSpecifics <- function(javaCmd,verbose) {
   if ( verbose ) cat("  ... querying Java specifics.\n")
-  response <- system2(path.expand(javaCmd),"-version",stdout=TRUE,stderr=TRUE)
+  response <- system2(javaCmd,"-version",stdout=TRUE,stderr=TRUE)
   # Get version information
   versionRegexp <- '(java|openjdk) version "([^"]*)".*'
   line <- response[grepl(versionRegexp,response)]
@@ -327,7 +342,7 @@ javaSpecifics <- function(javaCmd,verbose) {
   # Determine if 32 or 64 bit
   bit <- if ( any(grepl('^(Java HotSpot|OpenJDK).* 64-Bit (Server|Client) VM.*$',response)) ||
               any(grepl('^IBM .* amd64-64 .*$',response)) ) 64 else 32
-  list(javaCmd=normalizePath(javaCmd,mustWork=FALSE), javaMajorVersion=versionNumber, javaArchitecture=bit)
+  list(javaCmd=javaCmd, javaMajorVersion=versionNumber, javaArchitecture=bit)
 }
 
 setJavaEnv <- function(javaConf) {
@@ -349,7 +364,7 @@ scalaSpecifics <- function(scalaCmd,javaConf,verbose) {
   if ( verbose ) cat("  ... querying Scala specifics.\n")
   oldJavaEnv <- setJavaEnv(javaConf)
   info <- tryCatch({
-    system2(normalizePath(scalaCmd,mustWork=FALSE),c("-nobootcp","-nc","-e",shQuote('import util.Properties._; println(Seq(versionNumberString,scalaHome,javaHome).mkString(lineSeparator))')),stdout=TRUE,stderr=FALSE)
+    system2(scalaCmd,c("-nobootcp","-nc","-e",shQuote('import util.Properties._; println(Seq(versionNumberString,scalaHome,javaHome).mkString(lineSeparator))')),stdout=TRUE,stderr=FALSE)
    }, warning=function(e) "")
   setJavaEnv(oldJavaEnv)
   fullVersion <- info[1]
@@ -361,7 +376,7 @@ scalaSpecifics <- function(scalaCmd,javaConf,verbose) {
     if ( ( majorVersion == "2.11" ) && ( javaConf$javaMajorVersion > 8 ) ) {
       sprintf("Scala %s is not supported on Java %s.",majorVersion,javaConf$javaMajorVersion)
     } else {
-      list(scalaHome=normalizePath(info[2],mustWork=FALSE), scalaCmd=normalizePath(scalaCmd,mustWork=FALSE), scalaMajorVersion=majorVersion, scalaFullVersion=fullVersion, javaHome=normalizePath(info[3],mustWork=FALSE))
+      list(scalaHome=info[2], scalaCmd=scalaCmd, scalaMajorVersion=majorVersion, scalaFullVersion=fullVersion, javaHome=info[3])
     }
   }
 }
